// 003 u_dates
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_dates;
  interface
    uses SysUtils;

    const k_short_month: Array[1..12] of String[3]=
        ('jan', 'feb', 'mar', 'apr', 'may', 'jun',
         'jul', 'aug', 'sep', 'oct', 'nov', 'dec');

    function f_short_date(p_date_time: tDateTime): String;
    function f_year_and_month(p_date_time: tDateTime): String;
    function f_month_number_string(p_date_time: tDateTime): String;
    function f_year_and_month_number(p_date_time: tDateTime): String;

    function f_us_month_to_integer(p_month: String): Integer;
    function f_us_dd_mmm_yyyy(p_date_time: tDateTime; p_separator: Char): String;

    procedure set_short_us_month_names;
    function f_now_to_string: String;

  implementation

    function f_short_date(p_date_time: tDateTime): String;
      var l_day, l_month, l_year: Word;
      begin
        DecodeDate(p_date_time, l_year, l_month, l_day);
        Result:= Format('%2d-%3s-%4d', [l_day, k_short_month[l_month], l_year]);
      end; // f_short_date

    function f_year_and_month(p_date_time: tDateTime): String;
      var l_day, l_month, l_year: Word;
      begin
        DecodeDate(p_date_time, l_year, l_month, l_day);
        Result:= Format('%4d-%3s', [l_year, k_short_month[l_month]]);
      end; // f_year_and_month

    function f_month_number_string(p_date_time: tDateTime): String;
      begin
        Result:= FormatDateTime('mm', p_date_time);
      end; // f_month_number_string

    function f_year_and_month_number(p_date_time: tDateTime): String;
      var l_day, l_month, l_year: Word;
      begin
        DecodeDate(p_date_time, l_year, l_month, l_day);
        Result:= Format('%4d-', [l_year])+ f_month_number_string(p_date_time);
      end; // f_year_and_month_number

    function f_us_month_to_integer(p_month: String): Integer;
        // -- "jan" -> 1 or 0 if not a correct month
      var l_month: String;
      begin
        l_month:= LowerCase(p_month);

        Result:= 0;
        repeat
          Inc(Result);
        until (Result= 12) or (k_short_month[Result]= l_month);

        if (Result= 12) and (k_short_month[Result]<> l_month)
          then Result:= 0;
      end; // f_us_month_to_integer

    procedure set_short_us_month_names;
      var l_month_index: Integer;
      begin
        for l_month_index:= 1 to 12 do
          ShortMonthNames[l_month_index]:= k_short_month[l_month_index]
      end; // set_short_us_month_names

    function f_us_dd_mmm_yyyy(p_date_time: tDateTime; p_separator: Char): String;
      var l_year, l_month, l_day: Word;
      begin
        DecodeDate(p_date_time, l_year, l_month, l_day);
        Result:= Format('%2d', [l_day])+ p_separator+ k_short_month[l_month]+ p_separator+ IntToStr(l_year);
      end; // f_us_dd_mmm_yyyy

    function f_now_to_string: String;
      begin
        Result:= FormatDateTime('yyyy_mm_dd_hh_nn', Now);
      end; // f_now_to_string
      
    begin // u_dates
    end. // u_dates

















