// 004 u_c_display_packets
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- use a single wrapper unit to start / capture / close
// -- could be turned into a component

(*$r+*)

unit u_c_display_packets;
  interface
    uses Classes
        , Controls // tWinControl
        , ComCtrls // tPageControl, tListView
        , StdCtrls // tLabel
        , u_c_packet_sniffer
        , u_c_timer_now
        ;

    const k_list_packets_true= True;
          k_list_packets_false= False;
          k_collect_statistics_true= True;
          k_collect_statistics_false= False;

    type c_display_packets= class(c_packet_sniffer)
                              m_c_page_control: tPageControl;
                              m_c_stats_tab_sheet: tTabSheet;
                              m_c_line_list_view: tListView;
                              m_c_in_label, m_c_out_label, m_c_in_out_label,
                                  m_c_packet_count_label, m_c_timer_label: tLabel;
                              m_list_packets: Boolean;
                              m_c_stats_memo: tMemo;

                              m_c_timer_now: c_timer_now;

                              Constructor create_display_packets(p_name: String;
                                  p_c_owner: tWinControl);
                              procedure start_capture(p_adapter_index: Integer;
                                  p_collect_statistics: Boolean;
                                  p_file_name: String; p_list_packets: Boolean);
                              procedure add_list_view_line(p_id, p_in_or_out,
                                  p_protocol, p_sub_protocol, p_bytes,
                                  p_source_ip, p_source_port,
                                  p_destination_ip, p_destination_port: String);
                              procedure display_packet(p_c_packet_sniffer: c_packet_sniffer);
                              procedure stats_button_click(Sender: tObject);
                              Destructor Destroy; Override;
                            end; // c_display_packets

  implementation
    uses SysUtils
        , ExtCtrls // tPanel,
        , Graphics
        , u_c_display;

    // -- c_display_packets

    Constructor c_display_packets.create_display_packets(p_name: String;
        p_c_owner: tWinControl);

      procedure add_column(p_c_list_view: tListView; p_caption: String;
          p_width: Integer);
        begin
          with p_c_list_view.Columns.Add do
          begin
            Caption:= p_caption;
            Width:= p_width;
          end; // with p_c_list_view
        end; // add_column

      procedure create_label(var pv_c_label: tLabel;
          p_c_parent: tWinControl;
          var pv_left: Integer; p_top, p_width, p_height: Integer);
        begin
          pv_c_label:= tLabel.Create(p_c_owner);

          with pv_c_label do
          begin
            Parent:= p_c_parent;
            Visible:= True;
            Left:= pv_left; Top:= p_top; Width:= p_width; Height:= p_height;
            Color:= clRed;
          end;

          Inc(pv_left, p_width+ 10);
        end; // create_label

      const k_in_out_panel_width= 80;
      var l_c_stats_panel: tPanel;
          l_c_packets_tab_sheet: tTabSheet;
          l_running_x: Integer;
          l_c_button: tButton;

      begin // create_display_packets
        Inherited create_packet_sniffer(p_name);

        l_c_stats_panel:= tPanel.Create(p_c_owner);
        with l_c_stats_panel do
        begin
          Parent:= p_c_owner;
          Height:= 21+ 6;
          Align:= alTop;

          l_running_x:= 4;
          create_label(m_c_in_label, l_c_stats_panel, l_running_x, 4, k_in_out_panel_width, 13);
          create_label(m_c_out_label, l_c_stats_panel, l_running_x, 4, k_in_out_panel_width, 13);
          create_label(m_c_in_out_label, l_c_stats_panel, l_running_x, 4, k_in_out_panel_width, 13);
          create_label(m_c_packet_count_label, l_c_stats_panel, l_running_x, 4, k_in_out_panel_width, 13);
          create_label(m_c_timer_label, l_c_stats_panel, l_running_x, 4, k_in_out_panel_width, 13);

          l_c_button:= tButton.Create(p_c_owner);
          with l_c_button do
          begin
            Parent:= l_c_stats_panel;
            Left:= l_running_x+ 20;
            Top:= 2;
            Height:= 23;
            Caption:= 'stats_';
            OnClick:= stats_button_click;
          end;
        end; // with l_c_stats_panel

        m_c_page_control:= tPageControl.Create(p_c_owner);
        with m_c_page_control do
        begin
          Parent:= p_c_owner;
          Align:= alClient;

          l_c_packets_tab_sheet:= tTabSheet.Create(m_c_page_control);
          with l_c_packets_tab_sheet do
          begin
            Visible:= True;
            Caption:= 'packets';
            PageControl:= m_c_page_control;

            m_c_line_list_view:= tListView.Create(p_c_owner);
            with m_c_line_list_view do
            begin
              Parent:= l_c_packets_tab_sheet;

              Visible:= True;
              Align:= alClient;
              ViewStyle:= vsReport;

              add_column(m_c_line_list_view, 'id', 50);
              add_column(m_c_line_list_view, 'io', 20);
              add_column(m_c_line_list_view, 'proto', 60);
              add_column(m_c_line_list_view, 'sub_pro', 60);
              add_column(m_c_line_list_view, 'bytes', 60);
              add_column(m_c_line_list_view, 'source_ip', 130);
              add_column(m_c_line_list_view, 's_port', 70);
              add_column(m_c_line_list_view, 'destination_ip', 130);
              add_column(m_c_line_list_view, 'd_port', 70);
            end; // with m_c_line_list_view
          end; // with l_c_packets_tab_sheet

          m_c_stats_tab_sheet:= tTabSheet.Create(m_c_page_control);
          with m_c_stats_tab_sheet do
          begin
            Visible:= True;
            Caption:= 'stats';
            PageControl:= m_c_page_control;

            m_c_stats_memo:= tMemo.Create(p_c_owner);
            with m_c_stats_memo do
            begin
              Parent:= m_c_stats_tab_sheet;
              Align:= alClient;
              Font.Name:= 'Courier New';
              ScrollBars:= ssVertical;
            end;
          end; // with m_c_stats_tab_sheet

          ActivePage:= m_c_stats_tab_sheet;
          ActivePage:= l_c_packets_tab_sheet;
        end; // with l_c_page_control

        m_after_packet_arrived:= display_packet;

        m_c_timer_now:= c_timer_now.create_timer_now('timer');
      end; // create_display_packets

    procedure c_display_packets.start_capture(p_adapter_index: Integer;
        p_collect_statistics: Boolean;
        p_file_name: String; p_list_packets: Boolean);
      begin
        Inherited start_capture(p_adapter_index, p_collect_statistics, p_file_name);
        m_list_packets:= p_list_packets;
        m_c_timer_now.start_timer;
      end; // start_capture

    procedure c_display_packets.add_list_view_line(p_id, p_in_or_out,
        p_protocol, p_sub_protocol, p_bytes,
        p_source_ip, p_source_port,
        p_destination_ip, p_destination_port: String);
      begin
        with m_c_line_list_view, m_c_line_list_view.Items.Add do
        begin
          Caption:= p_id;
          SubItems.Add(p_in_or_out);
          SubItems.Add(p_protocol);
          SubItems.Add(p_sub_protocol);
          SubItems.Add(p_bytes);
          SubItems.Add(p_source_ip);
          SubItems.Add(p_source_port);
          SubItems.Add(p_destination_ip);
          SubItems.Add(p_destination_port);
        end;
      end; // add_list_view_line

    procedure c_display_packets.display_packet(p_c_packet_sniffer: c_packet_sniffer);
      var l_IP_source_addres, l_IP_source_port,
            l_IP_destination_address, l_IP_destination_port: String;
      begin
        // -- this somehow blocks the notebook switching
        // led_timer_.Enabled:= True;

        case m_direction of
          'i' : begin
                  m_c_in_label.Color:= clRed;
                  m_c_in_label.Caption:= Format(' in=%5d (%5d K)',
                     [m_input_packet_count, m_input_bytes div 1024]);
                end;
          'o' : begin
                  m_c_out_label.Color:= clRed;
                  m_c_out_label.Caption:= Format('out=%5d (%5d K)',
                       [m_output_packet_count, m_output_bytes div 1024]);
                end;
          else
            m_c_in_out_label.Color:= clRed;
            m_c_in_out_label.Caption:= Format('=%5d (%5d K)',
                [m_other_packet_count, m_other_bytes div 1024]);
        end; // case

        m_c_packet_count_label.Caption:= 'cnt ='+ IntToStr(m_input_packet_count
            + m_output_packet_count+ m_other_packet_count);

        m_c_timer_label.Caption:= m_c_timer_now.f_display_delta;    

        if m_IP_source_address<> ''
          then l_IP_source_addres:= m_IP_source_address
          else l_IP_source_addres:= '';
        if m_IP_source_port<> 0
          then l_IP_source_port:= IntToStr(m_IP_source_port)
          else l_IP_source_port:= '';
        if m_IP_destination_address<> ''
          then l_IP_destination_address:= m_IP_destination_address
          else l_IP_destination_address:= '';
        if m_IP_destination_port<> 0
          then l_IP_destination_port:= IntToStr(m_IP_destination_port)
          else l_IP_destination_port:= '';

        if m_list_packets
          then add_list_view_line(IntToStr(m_packet_number),
              m_direction, m_ethernet_protocol_family, m_sub_protocol_name,
              IntToStr(m_sub_protocol_size),
              l_IP_source_addres, l_IP_source_port,
              l_IP_destination_address, l_IP_destination_port);

        // Application.ProcessMessages;
      end; // display_packet

    procedure c_display_packets.stats_button_click(Sender: tObject);
      begin
        m_c_page_control.ActivePage:= m_c_stats_tab_sheet;

        display_stats(m_c_stats_memo.Lines);
      end; // stats_button_click

    Destructor c_display_packets.Destroy;
      begin
        m_c_timer_now.Free;
        
        Inherited;
      end; // Destroy

    begin // u_c_display_packets
    end. // u_c_display_packets
kets
