// 003 u_c_packet_sniffer
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- wrapper for c_sniffer, file log and statistical package

(*$r+*)

unit u_c_packet_sniffer;
  interface
    uses Classes // tStrings
        , ExtCtrls // tTimer
        , u_c_basic_object
        , u_c_sniffer
        , u_c_variable_record_file
        , u_c_packet_stats
        , u_ethernet_packet
        ;

    const k_collect_statistics_true= True;
          k_collect_statistics_false= False;
          k_no_log_file= '';

    type c_packet_sniffer= Class; // Forward;
         t_po_packet_arrived= Procedure(p_c_packet_sniffer: c_packet_sniffer) of Object;

         c_packet_sniffer= class(c_sniffer)
                             m_c_variable_record_file: c_variable_record_file;
                             m_packet_number: Integer;

                             m_c_packets_stats: c_packets_stats;

                             m_direction: Char;
                             m_sub_protocol_name: String;
                             m_sub_protocol_size: Integer;
                             m_ethernet_protocol_family: String;
                             m_IP_source_address, m_IP_destination_address: String;
                             m_IP_source_port, m_IP_destination_port: Integer;

                             m_input_packet_count, m_output_packet_count, m_other_packet_count: Integer;
                             m_input_bytes, m_output_bytes, m_other_bytes: Integer;

                             m_after_packet_arrived: t_po_packet_arrived;

                             Constructor create_packet_sniffer(p_name: String);

                             procedure analyze_packet(p_pt_ethernet_packet: t_pt_ethernet_packet; p_ethernet_packet_size: Integer);
                             procedure handle_packet(p_pt_packet: pointer; p_packet_size: Word);

                             procedure start_capture(p_adapter_index: Integer;
                                 p_collect_statistics: Boolean;
                                 p_file_name: String);
                             procedure stop_capture;
                             procedure reset_capture;

                             procedure display_stats(p_c_strings: tStrings);

                             Destructor Destroy; Override;
                           end; // c_packet_sniffer

  implementation
      uses SysUtils, u_c_display
          , u_dates, u_strings

          , u_sniffer_utils

          , u_ip_packet
          , u_ip_tcp_packet
          , u_ip_udp_packet
          , u_ip_icmp_packet

          ;

    // -- c_packet_sniffer

    Constructor c_packet_sniffer.create_packet_sniffer(p_name: String);
      var l_error: string;
      begin
        Inherited create_sniffer(p_name);

        if not f_get_adapters(l_error)
          then display_bug_stop('get_adapter: '+ l_error);
      end; // create_packet_sniffer

    procedure c_packet_sniffer.analyze_packet(p_pt_ethernet_packet: t_pt_ethernet_packet; p_ethernet_packet_size: Integer);

      procedure add_packets_to_stats(p_is_in_direction: Boolean;
          p_packet_stats_type: t_packet_stats_type; p_protocol_size: Integer);
        begin
          if m_c_packets_stats<> Nil
            then m_c_packets_stats.add_packet(p_is_in_direction, p_packet_stats_type, p_protocol_size);
        end; // add_packet_to_stats

      procedure analyze_ip_packet(p_pt_ip_packet: t_pt_ip_packet; p_ip_packet_size: Integer);

        procedure analyze_ip_tcp_packet(p_pt_ip_tcp_packet: t_pt_ip_tcp_packet; p_ip_tcp_packet_size: Integer);
          // var l_source_port, l_destination_port: Word;
          begin
            m_sub_protocol_name:= 'tcp';
            m_sub_protocol_size:= p_ip_tcp_packet_size- k_ip_tcp_header_size;

            with p_pt_ip_tcp_packet^ do
            begin
              m_IP_source_port:= f_swap_word(@ m_tcp_source_port);
              m_IP_destination_port:= f_swap_word(@ m_tcp_destination_port);
            end;

            add_packets_to_stats(m_direction= 'i', e_ip_tcp_packet, m_sub_protocol_size);
          end; // analyze_ip_tcp_packet

        procedure analyze_ip_udp_packet(p_pt_ip_udp_packet: t_pt_ip_udp_packet; p_ip_udp_packet_size: Integer);
          begin
            m_sub_protocol_name:= 'udp';
            m_sub_protocol_size:= p_ip_udp_packet_size- k_ip_udp_header_size;

            add_packets_to_stats(m_direction= 'i', e_ip_udp_packet, m_sub_protocol_size);
          end; // analyze_ip_udp_packet

        procedure analyze_ip_icmp_packet(p_pt_ip_icmp_packet: t_pt_ip_icmp_packet; p_ip_icmp_packet_size: Integer);
          begin
            m_sub_protocol_name:= 'icmp';
            m_sub_protocol_size:= p_ip_icmp_packet_size- k_ip_icmp_header_size;

            add_packets_to_stats(m_direction= 'i', e_ip_icmp_packet, m_sub_protocol_size);
          end; // analyze_ip_icmp_packet

        procedure swap_ips_and_ports;
          var l_save_ip: String;
              l_save_port: Integer;
          begin
            l_save_port:= m_IP_source_port;
            m_IP_source_port:= m_IP_destination_port;
            m_IP_destination_port:= l_save_port;
            l_save_ip:= m_IP_source_address;
            m_IP_source_address:= m_IP_destination_address;
            m_IP_destination_address:= l_save_ip;
          end; // swap_ips_and_ports

        var l_IP_sub_protocol_size: Integer;

        begin // analyze_ip_packet
          l_IP_sub_protocol_size:= p_ip_packet_size- k_ip_header_size;

          with p_pt_ip_packet^ do
          begin
            m_IP_source_address:= f_IP_to_string(m_IP_source);
            m_IP_destination_address:= f_IP_to_string(m_IP_destination);

            case m_ip_protocol of
              k_ip_tcp : analyze_ip_tcp_packet(t_pt_ip_tcp_packet(@ m_ip_data), l_IP_sub_protocol_size);
              k_ip_udp : analyze_ip_udp_packet(t_pt_ip_udp_packet(@ m_ip_data), l_IP_sub_protocol_size);
              k_ip_icmp : analyze_ip_icmp_packet(t_pt_ip_icmp_packet(@ m_ip_data), l_IP_sub_protocol_size);
              else
                m_sub_protocol_name:= '?';
                m_sub_protocol_size:= p_ip_packet_size;

                add_packets_to_stats(m_direction= 'i', e_ip_other_packet, m_sub_protocol_size);
            end; // case
          end; // with p_pt_ip_packet^

          if m_direction<> 'i'
            then swap_ips_and_ports;
        end; // analyze_ip_packet

      procedure compute_global_statistics;
        begin
          case m_direction of
            'i' : begin
                    Inc(m_input_packet_count);
                    Inc(m_input_bytes, p_ethernet_packet_size);
                  end;
            'o' : begin
                    Inc(m_output_packet_count);
                    Inc(m_output_bytes, p_ethernet_packet_size);
                  end;
            else
              Inc(m_other_packet_count);
              Inc(m_other_bytes, p_ethernet_packet_size);
          end; // case
        end; // compute_global_statistics

      var l_protocol_size: Integer;
          l_protocol: Word;

      begin // analyze_packet
        with p_pt_ethernet_packet^ do
        begin
          if CompareMem(@ g_MAC_address, @ m_ethernet_destination[0], 6)
            then m_direction:= k_reception // reception
            else
              if CompareMem(@ g_MAC_address, @ m_ethernet_source[0], 6)
                then m_direction:= k_emission // emission
                else m_direction:= k_no_direction;

          l_protocol_size:= p_ethernet_packet_size- k_ethernet_header_size;
          l_protocol:= f_swap_word(@ m_ethernet_protocol);
          m_ethernet_protocol_family:= f_protocol_family(l_protocol);
          m_sub_protocol_name:= '';
          m_sub_protocol_size:= l_protocol_size;
          m_IP_source_address:= ''; m_IP_destination_address:= '';
          m_IP_source_port:= 0; m_IP_destination_port:= 0;

          compute_global_statistics;

          case l_protocol of
            k_PPP_IPCP_protocol, k_PPP_LCP_protocol, k_PPP_PAP_protocol,
              k_PPP_CHAP_protocol, k_PPP_CCP_protocol,
              k_PPP_LQR_protocol :
                begin
                  add_packets_to_stats(m_direction= 'i', e_ppp_packet, m_sub_protocol_size);
                end;

            k_ARP_protocol :
                begin
                  add_packets_to_stats(m_direction= 'i', e_arp_packet, m_sub_protocol_size);
                end;
            k_IP_protocol : analyze_ip_packet(t_pt_ip_packet(@ m_ethernet_data), l_protocol_size);
            else
              add_packets_to_stats(m_direction= 'i', e_unknown_packet, m_sub_protocol_size);
          end; // case

          if Assigned(m_after_packet_arrived)
            then m_after_packet_arrived(Self);
        end; // with p_pt_ethernet_packet^  do
      end; // analyze_packet

    procedure c_packet_sniffer.handle_packet(p_pt_packet: pointer; p_packet_size: Word);
      begin
        // -- add the packet, whatever the protocol
        if m_c_variable_record_file<> Nil
          then m_c_variable_record_file.append_record(p_pt_packet, p_packet_size);
        Inc(m_packet_number);

        // -- display some sumary
        analyze_packet(t_pt_ethernet_packet(p_pt_packet), p_packet_size);
      end; // handle_packet

    procedure c_packet_sniffer.start_capture(p_adapter_index: Integer;
        p_collect_statistics: Boolean;
        p_file_name: String);
      var l_adapter_error: String;
      begin
        reset_capture;

        if p_collect_statistics
          then m_c_packets_stats:= c_packets_stats.create_packets_stats('stats');

        if p_file_name<> ''
          then begin
              // -- create the file
              m_c_variable_record_file.Free;
              m_c_variable_record_file:= c_variable_record_file.create_variable_record_file('var ', p_file_name);

              // -- if not exists
              if not FileExists(p_file_name)
                then begin
                    m_c_variable_record_file.f_force_create;
                    m_c_variable_record_file.close_file;

                    // -- keep the file open (speed)
                    m_c_variable_record_file.f_open;
                  end;
            end;

        // -- initialize the callback
        m_on_sniffer_received_packet:= handle_packet;

        // -- use this adapter
        m_current_adapter_index:= p_adapter_index;

        // -- start snooping
        if not f_activate(l_adapter_error)
          then raise Exception.create(l_adapter_error);
      end; // start_capture

    procedure c_packet_sniffer.stop_capture;
      var l_adapter_error: String;
      begin
        // -- if already running, stop the driver
        if m_is_snooping
          then
            if not f_deactivate(l_adapter_error)
              then raise Exception.create(l_adapter_error);

        if m_c_variable_record_file<> Nil
          then m_c_variable_record_file.close_file;
      end; // stop_capture

    procedure c_packet_sniffer.reset_capture;
      begin
        stop_capture;

        m_packet_number:= 0;

        m_input_packet_count:= 0; m_output_packet_count:= 0; m_other_packet_count:= 0;
        m_input_bytes:= 0; m_output_bytes:= 0; m_other_bytes:= 0;
      end; // reset_capture

    procedure c_packet_sniffer.display_stats(p_c_strings: tStrings);
      var l_packet_type: t_packet_stats_type;
          l_in, l_out: String;
          l_line_is_empty: Boolean;
      begin
        if m_c_packets_stats= Nil
          then begin
              display('*** did_not_capture_with_stats');
              exit;
            end;

        with p_c_strings, m_c_packets_stats do
        begin
          Clear;
          for l_packet_type:= e_arp_packet to e_total_packet do
          begin
            // display(IntToStr(f_packet_index(k_packet_in, l_packet_type)));
            l_line_is_empty:= True;
            with m_oa_stats[f_packet_index(k_packet_in, l_packet_type)] do
            begin
              l_in:= Format('%5d %12d  ', [m_count, m_bytes]);
              if m_count<> 0
                then l_line_is_empty:= False;
            end;
            with m_oa_stats[f_packet_index(k_packet_out, l_packet_type)] do
            begin
              l_out:= Format('%5d %12d  ', [m_count, m_bytes]);
              if m_count<> 0
                then l_line_is_empty:= False;
            end;

            if l_packet_type= e_total_packet
              then Add(Format('%-15s %-5s %-12s %-5s %-12s',
                  ['', f_characters(5, '-'), f_characters(12, '-'),
                  f_characters(5, '-'), f_characters(12, '-')]));
            if not l_line_is_empty
              then Add(Format('%-15s', [f_packet_name(l_packet_type)])+ l_in+ l_out);
          end; // for l_packet_type
        end; // with p_c_strings, m_c_packets_stats
      end; // display_stats

    Destructor c_packet_sniffer.Destroy;
      begin
        m_c_variable_record_file.Free;

        Inherited;
      end; // Destroy

    begin // u_c_packet_sniffer
    end. // u_c_packet_sniffer
ffer
