// 003 u_c_packet_stats
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- compute packet statisitics

(*$r+*)

unit u_c_packet_stats;
  interface
    uses Classes, StdCtrls, u_c_basic_object;

    const k_packet_format= '%5d %4d | ';
          k_packet_in= True;
          k_packet_out= False;

    type t_packet_stats_type= (
            e_arp_packet,
            e_ppp_packet,
            e_ip_icmp_packet,
            e_ip_tcp_packet,
            e_ip_udp_packet,
            e_ip_other_packet,
            e_unknown_packet,
            e_total_packet
            );

    function f_packet_name(p_packet_type: t_packet_stats_type): String;

    type c_packet_stats= class(c_basic_object)
                           m_count, m_bytes: Integer;
                           m_c_label: tLabel;

                           Constructor create_packet_stats(p_name: String;
                               p_c_label: tLabel);
                           procedure add_packet(p_bytes: Integer);
                           Destructor Destroy; Override;
                         end; // c_packet_stats

         c_packets_stats= class(c_basic_object)
                            m_oa_stats: array of c_packet_stats;

                            Constructor create_packets_stats(p_name: String); Virtual;
                            function f_packet_index(p_in_out: Boolean; p_packet_type: t_packet_stats_type): Integer;
                            procedure initialize_labels(p_packet_type: t_packet_stats_type;
                                p_c_label_in, p_c_label_out: tLabel);
                            procedure add_packet(p_in_out: Boolean;
                                p_packet_type: t_packet_stats_type; p_bytes: Integer);
                            // function  f_display_stats: String;
                            Destructor Destroy; Override;
                         end;

  implementation
    uses SysUtils, TypInfo, u_strings, u_c_display;

    function f_packet_name(p_packet_type: t_packet_stats_type): String;
      begin
        Result:= GetEnumName(TypeInfo(t_packet_stats_type), Integer(p_packet_type));
        Delete(Result, 1, 2);
        Delete(Result, Length(Result)- Length('_packet')+ 1, Length('_packet'));
      end; // f_packet_name

    // -- c_packet_stats

    Constructor c_packet_stats.create_packet_stats(p_name: String;
        p_c_label: tLabel);
      begin
        Inherited create_basic_object(p_name);
        m_c_label:= p_c_label;
      end; // create_packet_stats

    procedure c_packet_stats.add_packet(p_bytes: Integer);
      var l_line: String;
      begin
        Inc(m_count);
        Inc(m_bytes, p_bytes);

        if m_c_label<> Nil
          then begin
              l_line:= Format(k_packet_format, [m_count, m_bytes div 1024]);
              m_c_label.Caption:= l_line;
            end;
      end; // add_packet

    Destructor c_packet_stats.Destroy;
      begin
        Inherited;
      end; // Destroy

    // -- c_packets_stats

    Constructor c_packets_stats.create_packets_stats(p_name: String);
      var l_count, l_index: Integer;
      begin
        Inherited create_basic_object(p_name);

        l_count:= 2* (Ord(e_total_packet)+ 1);
        SetLength(m_oa_stats, l_count);

        for l_index:= 0 to l_count- 1 do
          m_oa_stats[l_index]:= c_packet_stats.create_packet_stats('', Nil);
      end; // create_packets_stats

    function c_packets_stats.f_packet_index(p_in_out: Boolean; p_packet_type: t_packet_stats_type): Integer;
      begin
        if p_in_out
          then Result:= 2* Ord(p_packet_type)
          else Result:= 2* Ord(p_packet_type)+ 1;
      end; // f_packet_index

    procedure c_packets_stats.initialize_labels(p_packet_type: t_packet_stats_type;
        p_c_label_in, p_c_label_out: tLabel);
      begin
        m_oa_stats[f_packet_index(k_packet_in, p_packet_type)].m_c_label:= p_c_label_in;
        m_oa_stats[f_packet_index(k_packet_out, p_packet_type)].m_c_label:= p_c_label_out;
      end; // initialize_labels

    procedure c_packets_stats.add_packet(p_in_out: Boolean;
        p_packet_type: t_packet_stats_type; p_bytes: Integer);
      begin
        m_oa_stats[f_packet_index(p_in_out, p_packet_type)].add_packet(p_bytes);
        m_oa_stats[f_packet_index(p_in_out, e_total_packet)].add_packet(p_bytes);
      end; // add_packet

    Destructor c_packets_stats.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_packet_stats
    end. // u_c_packet_stats

ts

